#!usr/bin/env python
"""

Usage:
    pnpm [-s | -d] <keywords>

Options:
    -h --help  显示帮助菜单
    -keywords  关键字
    -s         按star数排序
    -d         按下载量排序

Examples:
    pnpm lazyloading
    pnpm jquery-lazyloading
"""
from docopt import docopt
from npm_helper import BASE_URL, TABLE_HEADER, TABLE_ROW
from prettytable import PrettyTable
from spider_threads.entry import ThreadCreator
from spider_threads.utils import message
from .spiders.spider_list import NpmSearchSpider
from .spiders.spider_page import NpmPageSpider
from .data import database_creator


def get_keywords():
    arguments = docopt(__doc__, version="beta 0.1")
    keywords = arguments['<keywords>'].split('-')
    urls_generation(keywords)


def urls_generation(keywords):
    try:
        assert type(keywords) is list
        urls = [BASE_URL.format(query='+'.join(keywords), page=i) for i in range(1, 2)]
        thread_generation(urls)
    except AssertionError:
        message.error_message('except to receive a list')


def thread_generation(urls):
    threads = ThreadCreator(main_spider=NpmSearchSpider, branch_spider=NpmPageSpider)
    threads.get_entry_urls(urls)
    threads.finish_all_threads()
    print_table()


def print_table():
    database = database_creator()
    npm_table = PrettyTable(TABLE_HEADER)
    for index, package in enumerate(database.data):
        package["index"] = index
        package_row = [package[item] for item in TABLE_ROW]
        npm_table.add_row(package_row)
    print(npm_table)



#!usr/env/bin python
import threading

lock = threading.Lock()
database = None


def database_creator():
    """make sure you have only one instance of Database

    :return: DataBase's instance
    """
    global database
    if database is None:
        database = Database()
    return database


class Database(object):
    def __init__(self):
        self.data = []

    def append_data(self, data):
        lock.acquire()
        self.data.append(data)
        lock.release()





# -*- coding: UTF-8 -*-
"""
NPM HELPER
"""

__version__ = '0.1'

BASE_URL = 'https://www.npmjs.com/search?q={query}&page={page}'

TABLE_HEADER = ('编号', '名称', '简介', '月下载量')
TABLE_ROW = ('index', 'name', 'desc', 'download')



#!usr/bin/env python
import ssl
from urllib import request, error
from bs4 import BeautifulSoup
from spider_threads.utils import message

ssl._create_default_https_context = ssl._create_unverified_context


class NpmSearchSpider(object):
    def __init__(self, url):
        self.headers = {
            'Connection': 'Keep-Alive',
            'Accept': 'text/html, application/xhtml+xml, */*',
            'Accept-Language': 'en-US,en;q=0.8,zh-Hans-CN;q=0.5,zh-Hans;q=0.3',
            'User-Agent': 'Mozilla/5.0 (X11; U; Linux i686) Gecko/20071127 Firefox/2.0.0.11',
            'GET': url
        }
        self.url = url

    def request_urls(self):
        search_result = list()
        print('fetch {}'.format(self.url))
        try:
            req = request.Request(self.url, headers=self.headers)
            response = request.urlopen(req).read().decode('UTF-8')
            soup = BeautifulSoup(response, 'lxml')
            search_result = self.fetch_urls(soup, search_result)
        except (error.HTTPError, error.URLError, error.ContentTooShortError):
            message.error_message('http error or url error')
        except UnicodeEncodeError:
            message.error_message('encoding error')
        finally:
            return search_result

    @staticmethod
    def fetch_urls(soup, search_result):
        search_result_container = soup.find('ul', attrs={"class": "search-results"})
        search_results = search_result_container.find_all('li')
        if search_results:
            for result in search_results:
                result_head = result.find('h3').find('a', attrs={"class": "name"})
                result_url = result_head.attrs["href"]
                search_result.append(result_url)
        return search_result



#!usr/bin/env python
import ssl
import re
from urllib import request, error
from bs4 import BeautifulSoup
from spider_threads.utils import message
from ..data import database_creator

ssl._create_default_https_context = ssl._create_unverified_context

NPM_URL = 'https://www.npmjs.com{url}'
GITHUB = 'https://github.com{github}'


class NpmPageSpider(object):
    def __init__(self, url):
        url = NPM_URL.format(url=url)
        self.headers = {
            'Connection': 'Keep-Alive',
            'Accept': 'text/html, application/xhtml+xml, */*',
            'Accept-Language': 'en-US,en;q=0.8,zh-Hans-CN;q=0.5,zh-Hans;q=0.3',
            'User-Agent': 'Mozilla/5.0 (X11; U; Linux i686) Gecko/20071127 Firefox/2.0.0.11',
            'GET': url
        }
        self.url = url

    def request_page(self):
        print('fetch {}'.format(self.url))
        try:
            req = request.Request(self.url, headers=self.headers)
            response = request.urlopen(req).read().decode('UTF-8')
            soup = BeautifulSoup(response, 'lxml')
            self.fetch_page(soup)
        except (error.HTTPError, error.URLError, error.ContentTooShortError):
            message.error_message('http error or url error')
        except UnicodeEncodeError:
            message.error_message('encoding error')

    @staticmethod
    def fetch_page(soup):
        package = {}

        # fetch name & desc
        content = soup.find('div', attrs={"class": "content-column"})
        package_name_container = content.find('h1', attrs={"class": "package-name"})
        package_name = package_name_container.find('a').string.strip()
        package_desc = content.find('p', attrs={"class": "package-description"}).string.strip()
        package["name"] = package_name
        package["desc"] = package_desc[0:30:]

        # fetch sidebar info
        sidebar = soup.find('div', attrs={"class": "sidebar"})
        info_boxs = sidebar.find_all('ul', attrs={"class": "box"})
        if info_boxs:
            for index, info_box in enumerate(info_boxs):
                info_box_string = str(info_box)

                github_url = re.findall(r'>.*github.com(.*)</a>', info_box_string)
                if github_url:
                    package["github"] = GITHUB.format(github=github_url[0])
                monthly_download = re.search(r'monthly-downloads', info_box_string)
                if monthly_download:
                    downloads_number_container = info_box.find('strong', attrs={"class": "monthly-downloads"})
                    monthly_downloads = downloads_number_container.string.strip()
                    package["download"] = int(monthly_downloads)

        else:
            package["github"] = 'https://github.com/explore'
            package["download"] = 0

        database = database_creator()
        database.append_data(package)



HEADERS = {
    'Connection': 'Keep-Alive',
    'Accept': 'text/html, application/xhtml+xml, */*',
    'Accept-Language': 'en-US,en;q=0.8,zh-Hans-CN;q=0.5,zh-Hans;q=0.3',
    'User-Agent': 'Mozilla/5.0 (X11; U; Linux i686) Gecko/20071127 Firefox/2.0.0.11',
    'GET': None
}



from setuptools import setup, find_packages
from npm_helper import __version__

setup(
    name='npm_helper',
    version=__version__,
    keywords=('npm', 'search', 'github', 'spider'),
    description='npm website search is sooo silly and inconvenience. That\'s why this plugin born',
    author='ecmadao',
    author_email='wlec@outlook.com',
    url='https://github.com/ecmadao/npm-helper',
    packages=find_packages(),
    py_modules=['run'],
    include_package_data=True,
    platforms='any',
    install_requires=[
        'prettytable',
        'bs4',
        'spider_threads'
    ],
    entry_points={
        'console_scripts': ['pnpm=run:get_keywords']
    },
    license='MIT',
    zip_safe=False,
    classifiers=[
         'Environment :: Console',
         'Programming Language :: Python',
         'Programming Language :: Python :: 3.5',
         'Programming Language :: Python :: Implementation :: CPython'
    ]
)
